"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSegmentRunner = exports.useSeg = void 0;
const runtime_1 = require("@midwayjs/hooks/lib/runtime");
const _midway_1 = require("@midway");
const publish_1 = require("./publish");
const changefree_1 = require("./changefree");
const instanceLog_1 = require("../instanceLog");
const processInstance_1 = require("../processInstance");
const common_1 = require("../../common");
const instance_1 = require("../../typings/instance");
exports.useSeg = function useSeg() {
    const _req_ctx_ = this;
    const publish = async (params) => {
        var _a;
        const { segmentConfig, instanceContext, saveLog } = params;
        // 发布动作执行
        if (segmentConfig.changeFreeSwitch &&
            segmentConfig.env === 'prod' &&
            ((_a = instanceContext === null || instanceContext === void 0 ? void 0 : instanceContext.changeFreeCheckResult) === null || _a === void 0 ? void 0 : _a.check_status_enum) !== 'CHECK_PASS') {
            // 强制 ChangeFree 检查失败，线上环境
            throw new common_1.BizError('ChangeFree 检查未通过', 'CHANGEFREE_CHECK_FAILED');
        }
        await saveLog('开始进行发布调用，环境: ' + segmentConfig.env);
        const invokeResult = await _bind(publish_1.usePublishInvoke, _req_ctx_)(segmentConfig.invokeURI);
        if (!instanceContext.publishRecord) {
            instanceContext.publishRecord = {};
        }
        instanceContext.publishRecord[segmentConfig.env || 'unknown'] = Date.now(); // 上下文挂上最新发布时间戳
        await saveLog('发布调用完成，环境: ' + segmentConfig.env + ', 调用结果' + JSON.stringify(invokeResult));
        return {
            segmentDone: true,
            status: 'success',
            title: {
                local: '本地',
                daily: '日常',
                pre: '预发',
                prod: '线上',
                default: '未知',
            }[segmentConfig.env || 'default'] + '环境发布完成',
            actions: [
                {
                    type: 'primary',
                    content: '再发一次',
                    action: 'runOne',
                },
            ],
        };
    };
    const changefreeCheck = async (params) => {
        const { segmentConfig, instanceId, instanceContext, saveLog, userInfo, ctx } = params;
        // Changefree 变更风险检查
        const changeObjectFactory = (mode) => {
            if (mode === 'dev') {
                // 开发变更
                return {
                    change_type_key: 'CONFIG_PUSH',
                    change_object: `{"name":"轻店发布服务","config":"${instanceId}"}`,
                    change_object_type: 'CONFIG',
                };
            }
            else if (mode === 'op') {
                // 运营操作
                return {
                    change_type_key: 'OPERATIONAL_CHANGE',
                    change_object: `{"name":"轻店发布服务","content":"${instanceId}"}`,
                    change_object_type: 'STRATEGY',
                };
            }
            throw new common_1.BizError('ChangeFree模式非法', 'CHANGEFREE_MODE_INVALID');
        };
        const cfResult = await _bind(changefree_1.useChangeFreeCheck, _req_ctx_)({
            ...changeObjectFactory(segmentConfig.mode),
            source_order_id: String(instanceId),
            executor_emp_id: ctx.env === 'local' ? '176351' : (userInfo === null || userInfo === void 0 ? void 0 : userInfo.workid) || '',
            change_title: `轻店发布服务工单 ${instanceId}`,
            env: changefree_1.ALICHANGE_ENV_MAP[segmentConfig.env || 'prod'],
            change_start_time: String(Date.now()),
        });
        instanceContext.changeFreeCheckResult = cfResult;
        await saveLog(`${changefree_1.CHECK_STATUS_ENUM_HUMANMSG_MAP[cfResult === null || cfResult === void 0 ? void 0 : cfResult.check_status_enum] || '未知结果'}(${cfResult === null || cfResult === void 0 ? void 0 : cfResult.check_status_enum})`);
        if (cfResult.check_status_enum === 'CHECK_PASS') {
            return {
                segmentDone: true,
                status: 'success',
                title: '审批已通过，请继续发布',
                actions: [
                    {
                        type: 'normal',
                        action: 'url',
                        content: '查看审批单',
                        url: cfResult.apply_order_url,
                    },
                ],
            };
        }
        else if (cfResult.check_status_enum === 'CHECK_CANCEL') {
            return {
                segmentDone: false,
                status: 'error',
                title: '变更已被取消',
                actions: [],
            };
        }
        else if (cfResult.check_status_enum === 'CHECK_WAIT') {
            return {
                segmentDone: false,
                status: 'unknown',
                title: 'ChangeFree风险检测中',
                actions: [
                    {
                        type: 'primary',
                        action: 'runOne',
                        content: '刷新',
                    },
                ],
            };
        }
        else if (cfResult.check_status_enum === 'CHECK_REFUSE') {
            return {
                segmentDone: false,
                status: 'forbidden',
                title: '变更审批被拒绝，禁止变更',
                actions: [
                    {
                        type: 'primary',
                        action: 'fe-action',
                        content: '取消变更',
                        feAction: 'cancelChangefreeOrder',
                    },
                ],
            };
        }
        else if (cfResult.check_status_enum === 'CHECK_HOLD') {
            return {
                segmentDone: false,
                status: 'warn',
                title: '检测到风险，变更需要审批',
                actions: [
                    {
                        type: 'primary',
                        action: 'url',
                        content: '前往审批单',
                        url: cfResult.apply_order_url,
                    },
                    {
                        type: 'normal',
                        action: 'fe-action',
                        content: '重置变更',
                        feAction: 'cancelChangefreeOrder',
                    },
                ],
            };
        }
        else {
            return {
                segmentDone: false,
                status: 'unknown',
                title: 'ChangeFree检查遇到问题',
                actions: [
                    {
                        type: 'primary',
                        action: 'runOne',
                        content: '重试',
                    },
                ],
            };
        }
    };
    return {
        publish,
        changefreeCheck,
    };
};
exports.useSegmentRunner = function useSegmentRunner(processInstance, userInfo) {
    const _req_ctx_ = this;
    const ctx = _midway_1.useContext();
    const _segmentRunnerFuncMap = _bind(exports.useSeg, _req_ctx_)();
    let { curSegmentIndex, segmentList, context: instanceContext, status } = processInstance;
    const instanceId = Number(processInstance.id);
    const saveLog = (segmentName, description) => {
        // 记录流水工具函数
        return _bind(instanceLog_1.useNewLog, _req_ctx_)({
            processInstanceId: instanceId,
            segmentName: segmentName,
            description,
            context: JSON.stringify(instanceContext),
        });
    };
    const nextSegment = async () => {
        const nextSegmentIndex = curSegmentIndex + 1;
        if (nextSegmentIndex === segmentList.length) {
            // 流程执行完毕，结单
            const updateResult = await _bind(processInstance_1.useUpdateProcessInstance, _req_ctx_)({
                id: processInstance.id,
                status: instance_1.InstanceStatus.Done,
            });
            if (!updateResult) {
                throw new common_1.BizError('实例更新失败', 'INSTANCE_UPDATE_FAILED');
            }
            await saveLog('系统', '发布单实例执行完毕，结单');
            // 销毁缓存
            return;
        }
        // 流程继续进行，更新redis中context
        instanceContext = {
            processName: processInstance.processName,
            context: processInstance.context,
            segmentList: processInstance.segmentList,
            curSegmentIndex: nextSegmentIndex,
        };
        return;
    };
    const runSegment = async (isAutoRun = false, disableAutoNext = false) => {
        const curSegment = segmentList[curSegmentIndex];
        const segmentDesc = curSegment.desc || curSegment.name;
        if (isAutoRun && !curSegment.autoRun) {
            // 不允许自动执行
            return {
                segmentDone: false,
                status: 'info',
                title: segmentDesc + ' 需要手动执行',
                actions: [
                    {
                        type: 'primary',
                        content: '立即执行',
                        action: 'run',
                    },
                ],
            };
        }
        let segmentName = curSegment.name;
        let segmentConfig = curSegment.config;
        const runnerParams = {
            segment: curSegment,
            segmentName,
            segmentConfig,
            instanceId,
            instanceContext,
            userInfo,
            ctx,
            saveLog: (desc) => saveLog(segmentDesc, desc),
        };
        let result;
        try {
            if (!_segmentRunnerFuncMap[segmentName]) {
                throw new common_1.BizError('非法的流程环节', 'INVALID_SEGMENT_NAME');
            }
            result = await _segmentRunnerFuncMap[segmentName](runnerParams);
            if (!result) {
                throw new common_1.BizError('结果异常', 'INVALID_RUNNER_RESULT');
            }
            else if (result === null || result === void 0 ? void 0 : result.segmentDone) {
                const isLastSegment = curSegmentIndex + 1 === segmentList.length;
                const autoNext = curSegment.autoNext;
                if (isLastSegment) {
                    // 当前是最后一步，改状态
                    status = instance_1.InstanceStatus.Done;
                }
                else if (autoNext && !disableAutoNext) {
                    // 自动下一步
                    await nextSegment();
                    return await runSegment(true);
                }
                else {
                    // 手动下一步
                    result.actions = [
                        {
                            type: 'primary',
                            content: '下一步',
                            action: 'next',
                        },
                    ].concat(result.actions || []);
                }
            }
        }
        catch (error) {
            await saveLog(segmentName, `执行异常: [${error.code}]${error.message}`);
            result = {
                segmentDone: false,
                status: 'error',
                title: '流程执行异常',
                tips: `[${error.code}]${error.message}`,
                actions: [
                    {
                        type: 'primary',
                        content: '重试',
                        action: 'run',
                    },
                ],
            };
        }
        // 通用关闭按钮
        result.actions = (result.actions || []).concat([
            {
                type: 'normal',
                content: '关闭工单',
                action: 'close',
            },
        ]);
        return result;
    };
    return {
        status: status || -1,
        instanceContext,
        nextSegment,
        runSegment,
    };
};
//# sourceMappingURL=return.js.map